@@ -97,7 +97,6 @@ mod tests {
     #[test_case(&json!({"additionalProperties": false, "properties": {"foo": {}}, "patternProperties": {"f.*o": {"type": "integer"}}}), "additionalProperties: false")]
     #[test_case(&json!({"allOf": [{"type": "integer"}, {"minimum": 2}]}), "allOf: [{type: integer}, {minimum: 2}]")]
     #[test_case(&json!({"anyOf": [{"type": "integer"}, {"minimum": 2}]}), "anyOf: [{type: integer}, {minimum: 2}]")]
-    #[test_case(&json!(true), "true")]
     #[test_case(&json!(false), "false")]
     #[test_case(&json!({"const": 1}), "const: 1")]
     #[test_case(&json!({"contains": {"minimum": 5}}), "contains: {minimum: 5}")]
@@ -141,7 +140,7 @@ mod tests {
     #[test_case(&json!({"minimum": 1}), "minimum: 1")]
     #[test_case(&json!({"multipleOf": 1}), "multipleOf: 1")]
     #[test_case(&json!({"multipleOf": 1.5}), "multipleOf: 1.5")]
-    #[test_case(&json!({"not": true}), "not: true")]
+    #[test_case(&json!({"not": true}), "not: {}")]
     #[test_case(&json!({"oneOf": [{"type": "integer"}, {"minimum": 2}]}), "oneOf: [{type: integer}, {minimum: 2}]")]
     #[test_case(&json!({"pattern": "^a*$"}), "pattern: ^a*$")]
     #[test_case(&json!({"patternProperties": {"f.*o": {"type": "integer"}}}), "patternProperties: {f.*o: {type: integer}}")]
