@@ -14,19 +14,18 @@ pub(crate) struct PatternPropertiesValidator {
 
 impl PatternPropertiesValidator {
     #[inline]
-    pub(crate) fn compile(properties: &Value, context: &CompilationContext) -> CompilationResult {
-        if let Value::Object(map) = properties {
-            let mut patterns = Vec::with_capacity(map.len());
-            for (pattern, subschema) in map {
-                patterns.push((
-                    Regex::new(pattern)?,
-                    compile_validators(subschema, context)?,
-                ));
-            }
-            Ok(Box::new(PatternPropertiesValidator { patterns }))
-        } else {
-            Err(CompilationError::SchemaError)
+    pub(crate) fn compile(
+        map: &Map<String, Value>,
+        context: &CompilationContext,
+    ) -> CompilationResult {
+        let mut patterns = Vec::with_capacity(map.len());
+        for (pattern, subschema) in map {
+            patterns.push((
+                Regex::new(pattern)?,
+                compile_validators(subschema, context)?,
+            ));
         }
+        Ok(Box::new(PatternPropertiesValidator { patterns }))
     }
 }
 
@@ -61,12 +60,9 @@ impl Validate for PatternPropertiesValidator {
                     item.iter()
                         .filter(move |(key, _)| re.is_match(key))
                         .flat_map(move |(key, value)| {
+                            let instance_path = instance_path.push(key.to_owned());
                             validators.iter().flat_map(move |validator| {
-                                validator.validate(
-                                    schema,
-                                    value,
-                                    &instance_path.push(key.to_owned()),
-                                )
+                                validator.validate(schema, value, &instance_path)
                             })
                         })
                 })
@@ -91,6 +87,74 @@ impl ToString for PatternPropertiesValidator {
     }
 }
 
+pub(crate) struct SingleValuePatternPropertiesValidator {
+    pattern: Regex,
+    validators: Validators,
+}
+
+impl SingleValuePatternPropertiesValidator {
+    #[inline]
+    pub(crate) fn compile(
+        pattern: &str,
+        schema: &Value,
+        context: &CompilationContext,
+    ) -> CompilationResult {
+        Ok(Box::new(SingleValuePatternPropertiesValidator {
+            pattern: Regex::new(pattern)?,
+            validators: compile_validators(schema, context)?,
+        }))
+    }
+}
+
+impl Validate for SingleValuePatternPropertiesValidator {
+    fn is_valid(&self, schema: &JSONSchema, instance: &Value) -> bool {
+        if let Value::Object(item) = instance {
+            item.iter()
+                .filter(move |(key, _)| self.pattern.is_match(key))
+                .all(move |(_key, value)| {
+                    self.validators
+                        .iter()
+                        .all(move |validator| validator.is_valid(schema, value))
+                })
+        } else {
+            true
+        }
+    }
+
+    fn validate<'a>(
+        &self,
+        schema: &'a JSONSchema,
+        instance: &'a Value,
+        instance_path: &InstancePath,
+    ) -> ErrorIterator<'a> {
+        if let Value::Object(item) = instance {
+            let errors: Vec<_> = item
+                .iter()
+                .filter(move |(key, _)| self.pattern.is_match(key))
+                .flat_map(move |(key, value)| {
+                    let instance_path = instance_path.push(key.to_owned());
+                    self.validators.iter().flat_map(move |validator| {
+                        validator.validate(schema, value, &instance_path)
+                    })
+                })
+                .collect();
+            Box::new(errors.into_iter())
+        } else {
+            no_error()
+        }
+    }
+}
+
+impl ToString for SingleValuePatternPropertiesValidator {
+    fn to_string(&self) -> String {
+        format!(
+            "patternProperties: {{{}: {}}}",
+            self.pattern,
+            format_validators(&self.validators)
+        )
+    }
+}
+
 #[inline]
 pub(crate) fn compile(
     parent: &Map<String, Value>,
@@ -100,6 +164,19 @@ pub(crate) fn compile(
     match parent.get("additionalProperties") {
         // This type of `additionalProperties` validator handles `patternProperties` logic
         Some(Value::Bool(false)) | Some(Value::Object(_)) => None,
-        _ => Some(PatternPropertiesValidator::compile(schema, context)),
+        _ => {
+            if let Value::Object(map) = schema {
+                if map.len() == 1 {
+                    let (key, value) = map.iter().next().expect("Map is not empty");
+                    Some(SingleValuePatternPropertiesValidator::compile(
+                        key, value, context,
+                    ))
+                } else {
+                    Some(PatternPropertiesValidator::compile(map, context))
+                }
+            } else {
+                Some(Err(CompilationError::SchemaError))
+            }
+        }
     }
 }
