@@ -1,3 +1,4 @@
+use crate::error::error;
 use crate::{
     compilation::{context::CompilationContext, JSONSchema},
     error::{no_error, CompilationError, ErrorIterator, ValidationError},
@@ -13,20 +14,15 @@ pub(crate) struct RequiredValidator {
 
 impl RequiredValidator {
     #[inline]
-    pub(crate) fn compile(schema: &Value) -> CompilationResult {
-        match schema {
-            Value::Array(items) => {
-                let mut required = Vec::with_capacity(items.len());
-                for item in items {
-                    match item {
-                        Value::String(string) => required.push(string.clone()),
-                        _ => return Err(CompilationError::SchemaError),
-                    }
-                }
-                Ok(Box::new(RequiredValidator { required }))
+    pub(crate) fn compile(items: &[Value]) -> CompilationResult {
+        let mut required = Vec::with_capacity(items.len());
+        for item in items {
+            match item {
+                Value::String(string) => required.push(string.clone()),
+                _ => return Err(CompilationError::SchemaError),
             }
-            _ => Err(CompilationError::SchemaError),
         }
+        Ok(Box::new(RequiredValidator { required }))
     }
 }
 
@@ -72,11 +68,70 @@ impl ToString for RequiredValidator {
     }
 }
 
+pub(crate) struct SingleItemRequiredValidator {
+    value: String,
+}
+
+impl SingleItemRequiredValidator {
+    #[inline]
+    pub(crate) fn compile(value: &str) -> CompilationResult {
+        Ok(Box::new(SingleItemRequiredValidator {
+            value: value.to_string(),
+        }))
+    }
+}
+
+impl Validate for SingleItemRequiredValidator {
+    fn validate<'a>(
+        &self,
+        schema: &'a JSONSchema,
+        instance: &'a Value,
+        instance_path: &InstancePath,
+    ) -> ErrorIterator<'a> {
+        if !self.is_valid(schema, instance) {
+            return error(ValidationError::required(
+                instance_path.into(),
+                instance,
+                self.value.clone(),
+            ));
+        }
+        no_error()
+    }
+
+    fn is_valid(&self, _: &JSONSchema, instance: &Value) -> bool {
+        if let Value::Object(item) = instance {
+            item.contains_key(&self.value)
+        } else {
+            true
+        }
+    }
+}
+
+impl ToString for SingleItemRequiredValidator {
+    fn to_string(&self) -> String {
+        format!("required: [{}]", self.value)
+    }
+}
+
 #[inline]
 pub(crate) fn compile(
     _: &Map<String, Value>,
     schema: &Value,
     _: &CompilationContext,
 ) -> Option<CompilationResult> {
-    Some(RequiredValidator::compile(schema))
+    // IMPORTANT: If this function will ever return `None`, adjust `dependencies.rs` accordingly
+    match schema {
+        Value::Array(items) => {
+            if items.len() == 1 {
+                if let Some(Value::String(item)) = items.iter().next() {
+                    Some(SingleItemRequiredValidator::compile(item))
+                } else {
+                    Some(Err(CompilationError::SchemaError))
+                }
+            } else {
+                Some(RequiredValidator::compile(items))
+            }
+        }
+        _ => Some(Err(CompilationError::SchemaError)),
+    }
 }
